package com.ruoyi.web.controller.board;

import com.ruoyi.board.domain.PresetContent;
import com.ruoyi.board.service.IPresetContentService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 预置信息及模版Controller
 *
 * @author fuhao
 * @date 2024-08-12
 */
@RestController
@RequestMapping("/board/content")
public class PresetContentController extends BaseController
{
    @Autowired
    private IPresetContentService presetContentService;

    /**
     * 查询预置信息及模版列表
     */
    @PreAuthorize("@ss.hasPermi('board:content:list')")
    @GetMapping("/list")
    public TableDataInfo list(PresetContent presetContent)
    {
        startPage();
        List<PresetContent> list = presetContentService.list();
        return getDataTable(list);
    }

    /**
     * 导出预置信息及模版列表
     */
    @PreAuthorize("@ss.hasPermi('board:content:export')")
    @Log(title = "预置信息及模版", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PresetContent presetContent)
    {
        List<PresetContent> list = presetContentService.list();
        ExcelUtil<PresetContent> util = new ExcelUtil<PresetContent>(PresetContent.class);
        util.exportExcel(response, list, "预置信息及模版数据");
    }

    /**
     * 获取预置信息及模版详细信息
     */
    @PreAuthorize("@ss.hasPermi('board:content:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(presetContentService.getById(id));
    }

    /**
     * 新增预置信息及模版
     */
    @PreAuthorize("@ss.hasPermi('board:content:add')")
    @Log(title = "预置信息及模版", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PresetContent presetContent)
    {
        return toAjax(presetContentService.save(presetContent));
    }

    /**
     * 修改预置信息及模版
     */
    @PreAuthorize("@ss.hasPermi('board:content:edit')")
    @Log(title = "预置信息及模版", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody PresetContent presetContent)
    {
        return toAjax(presetContentService.updateById(presetContent));
    }

    /**
     * 删除预置信息及模版
     */
    @PreAuthorize("@ss.hasPermi('board:content:remove')")
    @Log(title = "预置信息及模版", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(presetContentService.removeByIds(Arrays.asList(ids)));
    }
}